#pragma rtGlobals=3
#pragma version=1.0
#pragma IgorVersion = 6.3.0
#pragma IndependentModule=CodeBrowserModule

// This file was created by () byte physics Thomas Braun, support@byte-physics.de
// (c) 2013

// Returns 1 for empty and null strings, zero otherwise
Function isEmpty(str)
	string &str

	return !(strlen(str) > 0)
End

// Add to every item in the list a prefix and/or suffix
Function/S addToItemsInList(list, [sep, suffix, prefix])
	string list, sep
	string suffix, prefix

	if(ParamIsDefault(sep))
		sep = ";"
	endif

	if(isEmpty(sep))
		Abort
	endif

	if(ParamIsDefault(suffix) && ParamIsDefault(prefix))
		return list
	endif

	if(ParamIsDefault(suffix))
		suffix = ""
	endif
	if(ParamIsDefault(prefix))
		prefix = ""
	endif

	if(strsearch(suffix,sep,0) != -1 || strsearch(prefix,sep,0) != -1)
		Abort
	endif

	variable i
	string resultList="", item

	variable numItems = ItemsInList(list,sep)
	for(i=0; i < numItems;i+=1)
		item = prefix + StringFromList(i,list) + suffix
		resultList = AddListItem(item,resultList,sep,inf)
	endfor

	return resultList
End

Function isProcGlobal(module)
	string module

	return cmpstr(module,"ProcGlobal") == 0
End

// Returns the dimension of the first screen
Function GetScreenDimensions(rect)
	STRUCT RECT &rect

	string str   = StringByKey("SCREEN1",IgorInfo(0))
	variable idx = strsearch(str,"RECT=",0)

	if(idx == -1)
		Abort "Unexpected information returned from IgorInfo(0)"
	endif

	str = str[idx+5,inf]

	if(ItemsInList(str,",") != 4)
		Abort "Could not find four values separated by ,"
	endif

	rect.left   = str2num(StringFromList(0,str,","))
	rect.top    = str2num(StringFromList(1,str,","))
	rect.right  = str2num(StringFromList(2,str,","))
	rect.bottom = str2num(StringFromList(3,str,","))
End

// Outputs a debug message prefixed with the calling function of debugPrint
Function debugPrint(msg)
	string msg

	if(debuggingEnabled)
		printf "%s(...): %s\r", GetRTStackInfo(2), RemoveEnding(msg,"\r")
	endif
End

// Creates a datafolder including all its parents and returns a datafolder
// reference to it
Function/DF createDFWithAllParents(dataFolder)
	string dataFolder

	variable i
	string partialPath="root"

	if(DataFolderExists(dataFolder))
		return $dataFolder
	endif

	 // i=1 because we want to skip root, as this exists always
	for(i=1; i < ItemsInList(dataFolder,":"); i+=1)
		partialPath += ":"
		partialPath += StringFromList(i,dataFolder,":")
		if(!DataFolderExists(partialPath))
			NewDataFolder $partialPath
		endif
	endfor

	return $dataFolder
end

// Returns from the full function declaration "myFunc()->(str) : <subtype>" only the name "myFunc"
Function/S getShortFuncOrMacroName(decl)
	string decl

	variable idx = strsearch(decl, "(", 0)
	if(idx > 0)
		return decl[0, idx-1]
	else
		return decl
	endif
End

// Searches for findStr in str and removes everything including the first match until the end
Function/S RemoveEverythingAfter(str,findStr)
	string str, findStr

	variable idx = strsearch(str,findStr,0)
	if(idx == -1)
		return str
	else
		return str[0,idx-1]
	endif
End